package geniusweb.domaineditor.model;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;

import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.DiscreteValueSet;
import geniusweb.issuevalue.ValueSet;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.mvc.model.BasicModel;
import tudelft.utilities.mvc.model.DefaultListModel;
import tudelft.utilities.mvc.model.ListModel;
import tudelft.utilities.mvc.model.StringModel;

/**
 * Model for a {@link DiscreteValueSet}. This is basically a
 * ListModel<StringModel>. However the list can never be empty, there must be
 * always at least 1 value.
 */
public class DiscreteValueSetModel extends DefaultListModel<StringModel>
		implements ValueSetModel {

	public DiscreteValueSetModel(List<StringModel> values, Reporter log) {
		super(values, log);
	}

	public DiscreteValueSetModel(Reporter log) {
		super(Arrays.asList(new StringModel("value", log)), log);
	}

	@Override
	public void remove(StringModel value) {
		if (getSize() == 1) {
			log.log(Level.WARNING,
					"remove not allowed, as value set always must contain at least 1 value");
			return;
		}
		super.remove(value);
	}

	/**
	 * 
	 * @return the {@link DiscreteValueSet} as set up by this model
	 */
	@Override
	public DiscreteValueSet getCurrentValue() {
		Collection<DiscreteValue> dvals = new LinkedList<>();
		for (int n = 0; n < getSize(); n++) {
			dvals.add(new DiscreteValue(get(n).getValue()));
		}
		return new DiscreteValueSet(dvals);
	}

	@Override
	public BasicModel first() {
		return get(0);
	}

	@Override
	public ListModel<StringModel> getList() {
		return this;
	}

	@Override
	public void setCurrentValue(ValueSet obj) throws IllegalArgumentException {
		if (!(obj instanceof DiscreteValueSet))
			throw new IllegalArgumentException(
					"Argument must be a DiscreteValueSet but got " + obj);
		DiscreteValueSet valset = (DiscreteValueSet) obj;
		List<StringModel> oldvals = new LinkedList<>();
		for (int n = 0; n < getSize(); n++) {
			oldvals.add(get(n));
		}

		// first add the new vals because minimum size.
		for (DiscreteValue val : valset.getValues()) {
			add(new StringModel(val.getValue(), log));
		}
		for (StringModel old : oldvals) {
			remove(old);
		}
	}

}
